from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M31SEV10


class BTMinerM31SEV10(M3X, M31SEV10):
    pass


from pyasic.miners.device.models import M31SEV20


class BTMinerM31SEV20(M3X, M31SEV20):
    pass


from pyasic.miners.device.models import M31SEV30


class BTMinerM31SEV30(M3X, M31SEV30):
    pass



from __future__ import annotations

from typing_extensions import Self

from pyasic.device.algorithm.hashrate.base import AlgoHashRateType
from pyasic.device.algorithm.hashrate.unit.equihash import EquihashUnit

from .unit import HashUnit


class EquihashHashRate(AlgoHashRateType):
    rate: float
    unit: EquihashUnit = HashUnit.ETHASH.default

    def into(self, other: EquihashUnit) -> Self:
        return self.__class__(
            rate=self.rate / (other.value / self.unit.value), unit=other
        )



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M56SVH30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M56SVH30

    expected_chips = 152
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M56SVJ30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M56SVJ30

    expected_chips = 132
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M56SVJ40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M56SVJ40

    expected_chips = 152
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M30SPlusV10


class BTMinerM30SPlusV10(M3X, M30SPlusV10):
    pass


from pyasic.miners.device.models import M30SPlusV100


class BTMinerM30SPlusV100(M3X, M30SPlusV100):
    pass


from pyasic.miners.device.models import M30SPlusV20


class BTMinerM30SPlusV20(M3X, M30SPlusV20):
    pass


from pyasic.miners.device.models import M30SPlusV30


class BTMinerM30SPlusV30(M3X, M30SPlusV30):
    pass


from pyasic.miners.device.models import M30SPlusV40


class BTMinerM30SPlusV40(M3X, M30SPlusV40):
    pass


from pyasic.miners.device.models import M30SPlusV50


class BTMinerM30SPlusV50(M3X, M30SPlusV50):
    pass


from pyasic.miners.device.models import M30SPlusV60


class BTMinerM30SPlusV60(M3X, M30SPlusV60):
    pass


from pyasic.miners.device.models import M30SPlusV70


class BTMinerM30SPlusV70(M3X, M30SPlusV70):
    pass


from pyasic.miners.device.models import M30SPlusV80


class BTMinerM30SPlusV80(M3X, M30SPlusV80):
    pass


from pyasic.miners.device.models import M30SPlusV90


class BTMinerM30SPlusV90(M3X, M30SPlusV90):
    pass


from pyasic.miners.device.models import M30SPlusVE100


class BTMinerM30SPlusVE100(M3X, M30SPlusVE100):
    pass


from pyasic.miners.device.models import M30SPlusVE30


class BTMinerM30SPlusVE30(M3X, M30SPlusVE30):
    pass


from pyasic.miners.device.models import M30SPlusVE40


class BTMinerM30SPlusVE40(M3X, M30SPlusVE40):
    pass


from pyasic.miners.device.models import M30SPlusVE50


class BTMinerM30SPlusVE50(M3X, M30SPlusVE50):
    pass


from pyasic.miners.device.models import M30SPlusVE60


class BTMinerM30SPlusVE60(M3X, M30SPlusVE60):
    pass


from pyasic.miners.device.models import M30SPlusVE70


class BTMinerM30SPlusVE70(M3X, M30SPlusVE70):
    pass


from pyasic.miners.device.models import M30SPlusVE80


class BTMinerM30SPlusVE80(M3X, M30SPlusVE80):
    pass


from pyasic.miners.device.models import M30SPlusVE90


class BTMinerM30SPlusVE90(M3X, M30SPlusVE90):
    pass


from pyasic.miners.device.models import M30SPlusVF20


class BTMinerM30SPlusVF20(M3X, M30SPlusVF20):
    pass


from pyasic.miners.device.models import M30SPlusVF30


class BTMinerM30SPlusVF30(M3X, M30SPlusVF30):
    pass


from pyasic.miners.device.models import M30SPlusVG20


class BTMinerM30SPlusVG20(M3X, M30SPlusVG20):
    pass


from pyasic.miners.device.models import M30SPlusVG30


class BTMinerM30SPlusVG30(M3X, M30SPlusVG30):
    pass


from pyasic.miners.device.models import M30SPlusVG40


class BTMinerM30SPlusVG40(M3X, M30SPlusVG40):
    pass


from pyasic.miners.device.models import M30SPlusVG50


class BTMinerM30SPlusVG50(M3X, M30SPlusVG50):
    pass


from pyasic.miners.device.models import M30SPlusVG60


class BTMinerM30SPlusVG60(M3X, M30SPlusVG60):
    pass


from pyasic.miners.device.models import M30SPlusVH10


class BTMinerM30SPlusVH10(M3X, M30SPlusVH10):
    pass


from pyasic.miners.device.models import M30SPlusVH20


class BTMinerM30SPlusVH20(M3X, M30SPlusVH20):
    pass


from pyasic.miners.device.models import M30SPlusVH30


class BTMinerM30SPlusVH30(M3X, M30SPlusVH30):
    pass


from pyasic.miners.device.models import M30SPlusVH40


class BTMinerM30SPlusVH40(M3X, M30SPlusVH40):
    pass


from pyasic.miners.device.models import M30SPlusVH50


class BTMinerM30SPlusVH50(M3X, M30SPlusVH50):
    pass


from pyasic.miners.device.models import M30SPlusVH60


class BTMinerM30SPlusVH60(M3X, M30SPlusVH60):
    pass


from pyasic.miners.device.models import M30SPlusVH70


class BTMinerM30SPlusVH70(M3X, M30SPlusVH70):
    pass


from pyasic.miners.device.models import M30SPlusVI30


class BTMinerM30SPlusVI30(M3X, M30SPlusVI30):
    pass


from pyasic.miners.device.models import M30SPlusVJ30


class BTMinerM30SPlusVJ30(M3X, M30SPlusVJ30):
    pass


from pyasic.miners.device.models import M30SPlusVJ40


class BTMinerM30SPlusVJ40(M3X, M30SPlusVJ40):
    pass



from pyasic.miners.backends.iceriver import IceRiver
from pyasic.miners.device.models.iceriver import KS5, KS5L, KS5M


class IceRiverKS5(IceRiver, KS5):
    pass


class IceRiverKS5L(IceRiver, KS5L):
    pass


class IceRiverKS5M(IceRiver, KS5M):
    pass



from pyasic.miners.backends.iceriver import IceRiver
from pyasic.miners.device.models.iceriver import KS3, KS3L, KS3M


class IceRiverKS3(IceRiver, KS3):
    pass


class IceRiverKS3L(IceRiver, KS3L):
    pass


class IceRiverKS3M(IceRiver, KS3M):
    pass



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import KHeavyHashHashRate
from .hashrate.unit import KHeavyHashUnit


# make this json serializable
class KHeavyHashAlgo(MinerAlgoType):
    hashrate: type[KHeavyHashHashRate] = KHeavyHashHashRate
    unit: type[KHeavyHashUnit] = KHeavyHashUnit

    name = "KHeavyHash"



from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M56SPlusPlusVK10


class BTMinerM56SPlusPlusVK10(M5X, M56SPlusPlusVK10):
    pass


from pyasic.miners.device.models import M56SPlusPlusVK30


class BTMinerM56SPlusPlusVK30(M5X, M56SPlusPlusVK30):
    pass


from pyasic.miners.device.models import M56SPlusPlusVK40


class BTMinerM56SPlusPlusVK40(M5X, M56SPlusPlusVK40):
    pass


from pyasic.miners.device.models import M56SPlusPlusVK50


class BTMinerM56SPlusPlusVK50(M5X, M56SPlusPlusVK50):
    pass



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M63SPlusVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SPlusVK30

    expected_chips = 456
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SPlusVL10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SPlusVL10

    expected_chips = 304
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SPlusVL20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SPlusVL20

    expected_chips = 340
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SPlusVL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SPlusVL30

    expected_chips = 370
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SPlusVL50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SPlusVL50

    expected_chips = 272
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



from .M20 import BTMinerM20V10
from .M20P import BTMinerM20PV10, BTMinerM20PV30
from .M20S import BTMinerM20SV10, BTMinerM20SV20, BTMinerM20SV30
from .M20S_Plus import BTMinerM20SPlusV30
from .M21 import BTMinerM21V10
from .M21S import BTMinerM21SV20, BTMinerM21SV60, BTMinerM21SV70
from .M21S_Plus import BTMinerM21SPlusV20
from .M29 import BTMinerM29V10



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import BraiinsMake


class BMM100(BraiinsMake):
    raw_model = MinerModel.BRAIINS.BMM100

    expected_hashboards = 1
    expected_fans = 1
    algo = MinerAlgo.SHA256


class BMM101(BraiinsMake):
    raw_model = MinerModel.BRAIINS.BMM101

    expected_hashboards = 1
    expected_fans = 1
    algo = MinerAlgo.SHA256



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import KadenaHashRate
from .hashrate.unit import KadenaUnit


# make this json serializable
class KadenaAlgo(MinerAlgoType):
    hashrate: type[KadenaHashRate] = KadenaHashRate
    unit: type[KadenaUnit] = KadenaUnit

    name = "Kadena"



from enum import IntEnum


class AlgoHashRateUnitType(IntEnum):
    H: int
    KH: int
    MH: int
    GH: int
    TH: int
    PH: int
    EH: int
    ZH: int

    default: int

    def __str__(self):
        if self.value == self.H:
            return "H/s"
        if self.value == self.KH:
            return "KH/s"
        if self.value == self.MH:
            return "MH/s"
        if self.value == self.GH:
            return "GH/s"
        if self.value == self.TH:
            return "TH/s"
        if self.value == self.PH:
            return "PH/s"
        if self.value == self.EH:
            return "EH/s"
        if self.value == self.ZH:
            return "ZH/s"

    @classmethod
    def from_str(cls, value: str):
        if value == "H":
            return cls.H
        elif value == "KH":
            return cls.KH
        elif value == "MH":
            return cls.MH
        elif value == "GH":
            return cls.GH
        elif value == "TH":
            return cls.TH
        elif value == "PH":
            return cls.PH
        elif value == "EH":
            return cls.EH
        elif value == "ZH":
            return cls.ZH
        return cls.default

    def __repr__(self):
        return str(self)

    def model_dump(self):
        return {"value": self.value, "suffix": str(self)}


class GenericUnit(AlgoHashRateUnitType):
    H = 1
    KH = int(H) * 1000
    MH = int(KH) * 1000
    GH = int(MH) * 1000
    TH = int(GH) * 1000
    PH = int(TH) * 1000
    EH = int(PH) * 1000
    ZH = int(EH) * 1000

    default = H



from .M20 import M20V10
from .M20P import M20PV10, M20PV30
from .M20S import M20SV10, M20SV20, M20SV30
from .M20S_Plus import M20SPlusV30
from .M21 import M21V10
from .M21S import M21SV20, M21SV60, M21SV70
from .M21S_Plus import M21SPlusV20
from .M29 import M29V10



from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M63SVK10


class BTMinerM63SVK10(M6X, M63SVK10):
    pass


from pyasic.miners.device.models import M63SVK20


class BTMinerM63SVK20(M6X, M63SVK20):
    pass


from pyasic.miners.device.models import M63SVK30


class BTMinerM63SVK30(M6X, M63SVK30):
    pass


from pyasic.miners.device.models import M63SVK60


class BTMinerM63SVK60(M6X, M63SVK60):
    pass


from pyasic.miners.device.models import M63SVL10


class BTMinerM63SVL10(M6X, M63SVL10):
    pass


from pyasic.miners.device.models import M63SVL50


class BTMinerM63SVL50(M6X, M63SVL50):
    pass


from pyasic.miners.device.models import M63SVL60


class BTMinerM63SVL60(M6X, M63SVL60):
    pass



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import X11HashRate
from .hashrate.unit import X11Unit


# make this json serializable
class X11Algo(MinerAlgoType):
    hashrate: type[X11HashRate] = X11HashRate
    unit: type[X11Unit] = X11Unit

    name = "X11"



from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M65SVK20


class BTMinerM65SVK20(M6X, M65SVK20):
    pass


from pyasic.miners.device.models import M65SVL60


class BTMinerM65SVL60(M6X, M65SVL60):
    pass



from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M53SPlusPlusVK10


class BTMinerM53SPlusPlusVK10(M5X, M53SPlusPlusVK10):
    pass


from pyasic.miners.device.models import M53SPlusPlusVK20


class BTMinerM53SPlusPlusVK20(M5X, M53SPlusPlusVK20):
    pass


from pyasic.miners.device.models import M53SPlusPlusVK30


class BTMinerM53SPlusPlusVK30(M5X, M53SPlusPlusVK30):
    pass


from pyasic.miners.device.models import M53SPlusPlusVK50


class BTMinerM53SPlusPlusVK50(M5X, M53SPlusPlusVK50):
    pass


from pyasic.miners.device.models import M53SPlusPlusVL10


class BTMinerM53SPlusPlusVL10(M5X, M53SPlusPlusVL10):
    pass


from pyasic.miners.device.models import M53SPlusPlusVL30


class BTMinerM53SPlusPlusVL30(M5X, M53SPlusPlusVL30):
    pass



# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------
from __future__ import annotations

from dataclasses import field
from typing import TypeVar, Union

from pyasic import settings
from pyasic.config.base import MinerConfigOption, MinerConfigValue
from pyasic.web.braiins_os.proto.braiins.bos.v1 import (
    DpsHashrateTarget,
    DpsPowerTarget,
    DpsTarget,
    HashrateTargetMode,
    PerformanceMode,
    Power,
    PowerTargetMode,
    SaveAction,
    SetDpsRequest,
    SetPerformanceModeRequest,
    TeraHashrate,
    TunerPerformanceMode,
)

from .algo import TunerAlgo, TunerAlgoType
from .presets import MiningPreset
from .scaling import ScalingConfig


class MiningModeNormal(MinerConfigValue):
    mode: str = field(init=False, default="normal")

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> "MiningModeNormal":
        return cls()

    def as_am_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_hiveon_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_elphapex(self) -> dict:
        return {"miner-mode": 0}

    def as_wm(self) -> dict:
        return {"mode": self.mode}

    def as_auradine(self) -> dict:
        return {"mode": {"mode": self.mode}}

    def as_epic(self) -> dict:
        return {"ptune": {"enabled": False}}

    def as_goldshell(self) -> dict:
        return {"settings": {"level": 0}}

    def as_mara(self) -> dict:
        return {
            "mode": {
                "work-mode-selector": "Stock",
            }
        }

    def as_luxos(self) -> dict:
        return {"autotunerset": {"enabled": False}}


class MiningModeSleep(MinerConfigValue):
    mode: str = field(init=False, default="sleep")

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> "MiningModeSleep":
        return cls()

    def as_am_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "1"}
        return {"miner-mode": 1}

    def as_hiveon_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "1"}
        return {"miner-mode": 1}

    def as_elphapex(self) -> dict:
        return {"miner-mode": 1}

    def as_wm(self) -> dict:
        return {"mode": self.mode}

    def as_auradine(self) -> dict:
        return {"mode": {"sleep": "on"}}

    def as_epic(self) -> dict:
        return {"ptune": {"algo": "Sleep", "target": 0}}

    def as_goldshell(self) -> dict:
        return {"settings": {"level": 3}}

    def as_mara(self) -> dict:
        return {
            "mode": {
                "work-mode-selector": "Sleep",
            }
        }


class MiningModeLPM(MinerConfigValue):
    mode: str = field(init=False, default="low")

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> "MiningModeLPM":
        return cls()

    def as_am_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "3"}
        return {"miner-mode": 3}

    def as_hiveon_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "3"}
        return {"miner-mode": 3}

    def as_elphapex(self) -> dict:
        return {"miner-mode": 3}

    def as_wm(self) -> dict:
        return {"mode": self.mode}

    def as_auradine(self) -> dict:
        return {"mode": {"mode": "eco"}}

    def as_goldshell(self) -> dict:
        return {"settings": {"level": 1}}


class MiningModeHPM(MinerConfigValue):
    mode: str = field(init=False, default="high")

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> "MiningModeHPM":
        return cls()

    def as_am_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_hiveon_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_elphapex(self) -> dict:
        return {"miner-mode": 0}

    def as_wm(self) -> dict:
        return {"mode": self.mode}

    def as_auradine(self) -> dict:
        return {"mode": {"mode": "turbo"}}


class MiningModePowerTune(MinerConfigValue):
    class Config:
        arbitrary_types_allowed = True

    mode: str = field(init=False, default="power_tuning")
    power: int | None = None
    algo: TunerAlgoType = field(default_factory=TunerAlgo.default)
    scaling: ScalingConfig | None = None

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> "MiningModePowerTune":
        cls_conf = {}
        if dict_conf.get("power"):
            cls_conf["power"] = dict_conf["power"]
        if dict_conf.get("algo"):
            cls_conf["algo"] = TunerAlgo.from_dict(dict_conf["algo"])
        if dict_conf.get("scaling"):
            cls_conf["scaling"] = ScalingConfig.from_dict(dict_conf["scaling"])

        return cls(**cls_conf)

    def as_am_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_hiveon_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_elphapex(self) -> dict:
        return {"miner-mode": 0}

    def as_wm(self) -> dict:
        if self.power is not None:
            return {"mode": self.mode, self.mode: {"wattage": self.power}}
        return {}

    def as_bosminer(self) -> dict:
        tuning_cfg = {"enabled": True, "mode": "power_target"}
        if self.power is not None:
            tuning_cfg["power_target"] = self.power

        cfg = {"autotuning": tuning_cfg}

        if self.scaling is not None:
            scaling_cfg = {"enabled": True}
            if self.scaling.step is not None:
                scaling_cfg["power_step"] = self.scaling.step
            if self.scaling.minimum is not None:
                scaling_cfg["min_power_target"] = self.scaling.minimum
            if self.scaling.shutdown is not None:
                scaling_cfg = {**scaling_cfg, **self.scaling.shutdown.as_bosminer()}
            cfg["performance_scaling"] = scaling_cfg

        return cfg

    def as_boser(self) -> dict:
        cfg = {
            "set_performance_mode": SetPerformanceModeRequest(
                save_action=SaveAction.SAVE_AND_APPLY,
                mode=PerformanceMode(
                    tuner_mode=TunerPerformanceMode(
                        power_target=PowerTargetMode(
                            power_target=Power(watt=self.power)
                        )
                    )
                ),
            ),
        }
        if self.scaling is not None:
            sd_cfg = {}
            if self.scaling.shutdown is not None:
                sd_cfg = self.scaling.shutdown.as_boser()
            cfg["set_dps"] = (
                SetDpsRequest(
                    enable=True,
                    **sd_cfg,
                    target=DpsTarget(
                        power_target=DpsPowerTarget(
                            power_step=Power(self.scaling.step),
                            min_power_target=Power(self.scaling.minimum),
                        )
                    ),
                ),
            )
        return cfg

    def as_auradine(self) -> dict:
        return {"mode": {"mode": "custom", "tune": "power", "power": self.power}}

    def as_mara(self) -> dict:
        return {
            "mode": {
                "work-mode-selector": "Auto",
                "concorde": {
                    "mode-select": "PowerTarget",
                    "power-target": self.power,
                },
            }
        }

    def as_luxos(self) -> dict:
        return {"autotunerset": {"enabled": True}}


class MiningModeHashrateTune(MinerConfigValue):
    class Config:
        arbitrary_types_allowed = True

    mode: str = field(init=False, default="hashrate_tuning")
    hashrate: int | None = None
    algo: TunerAlgoType = field(default_factory=TunerAlgo.default)
    scaling: ScalingConfig | None = None

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> "MiningModeHashrateTune":
        cls_conf = {}
        if dict_conf.get("hashrate"):
            cls_conf["hashrate"] = dict_conf["hashrate"]
        if dict_conf.get("algo"):
            cls_conf["algo"] = TunerAlgo.from_dict(dict_conf["algo"])
        if dict_conf.get("scaling"):
            cls_conf["scaling"] = ScalingConfig.from_dict(dict_conf["scaling"])

        return cls(**cls_conf)

    def as_am_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_hiveon_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_elphapex(self) -> dict:
        return {"miner-mode": 0}

    def as_bosminer(self) -> dict:
        conf = {"enabled": True, "mode": "hashrate_target"}
        if self.hashrate is not None:
            conf["hashrate_target"] = self.hashrate
        return {"autotuning": conf}

    @property
    def as_boser(self) -> dict:
        cfg = {
            "set_performance_mode": SetPerformanceModeRequest(
                save_action=SaveAction.SAVE_AND_APPLY,
                mode=PerformanceMode(
                    tuner_mode=TunerPerformanceMode(
                        hashrate_target=HashrateTargetMode(
                            hashrate_target=TeraHashrate(
                                terahash_per_second=self.hashrate
                            )
                        )
                    )
                ),
            )
        }
        if self.scaling is not None:
            sd_cfg = {}
            if self.scaling.shutdown is not None:
                sd_cfg = self.scaling.shutdown.as_boser()
            cfg["set_dps"] = (
                SetDpsRequest(
                    enable=True,
                    **sd_cfg,
                    target=DpsTarget(
                        hashrate_target=DpsHashrateTarget(
                            hashrate_step=TeraHashrate(self.scaling.step),
                            min_hashrate_target=TeraHashrate(self.scaling.minimum),
                        )
                    ),
                ),
            )
        return cfg

    def as_auradine(self) -> dict:
        return {"mode": {"mode": "custom", "tune": "ths", "ths": self.hashrate}}

    def as_epic(self) -> dict:
        mode = {
            "ptune": {
                "algo": self.algo.as_epic(),
                "target": self.hashrate,
            }
        }
        if self.scaling is not None:
            if self.scaling.minimum is not None:
                mode["ptune"]["min_throttle"] = self.scaling.minimum
            if self.scaling.step is not None:
                mode["ptune"]["throttle_step"] = self.scaling.step
        return mode

    def as_mara(self) -> dict:
        return {
            "mode": {
                "work-mode-selector": "Auto",
                "concorde": {
                    "mode-select": "Hashrate",
                    "hash-target": self.hashrate,
                },
            }
        }

    def as_luxos(self) -> dict:
        return {"autotunerset": {"enabled": True}}


class MiningModePreset(MinerConfigValue):
    mode: str = field(init=False, default="preset")

    active_preset: MiningPreset
    available_presets: list[MiningPreset] = field(default_factory=list)

    def as_vnish(self) -> dict:
        return {"overclock": {**self.active_preset.as_vnish()}}

    @classmethod
    def from_vnish(
        cls, web_overclock_settings: dict, web_presets: list[dict]
    ) -> "MiningModePreset":
        active_preset = None
        for preset in web_presets:
            if preset["name"] == web_overclock_settings["preset"]:
                active_preset = preset
        return cls(
            active_preset=MiningPreset.from_vnish(active_preset),
            available_presets=[MiningPreset.from_vnish(p) for p in web_presets],
        )

    @classmethod
    def from_luxos(
        cls, rpc_config: dict, rpc_profiles: list[dict]
    ) -> "MiningModePreset":
        active_preset = cls.get_active_preset_from_luxos(rpc_config, rpc_profiles)
        return cls(
            active_preset=MiningPreset.from_luxos(active_preset),
            available_presets=[
                MiningPreset.from_luxos(p) for p in rpc_profiles["PROFILES"]
            ],
        )

    @classmethod
    def get_active_preset_from_luxos(
        cls, rpc_config: dict, rpc_profiles: list[dict]
    ) -> dict:
        active_preset = None
        active_profile = rpc_config["CONFIG"][0]["Profile"]
        for profile in rpc_profiles["PROFILES"]:
            if profile["Profile Name"] == active_profile:
                active_preset = profile
        return active_preset


class ManualBoardSettings(MinerConfigValue):
    freq: float
    volt: float

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> "ManualBoardSettings":
        return cls(freq=dict_conf["freq"], volt=dict_conf["volt"])

    def as_am_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_hiveon_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_elphapex(self) -> dict:
        return {"miner-mode": 0}

    def as_vnish(self) -> dict:
        return {"freq": self.freq}


class MiningModeManual(MinerConfigValue):
    mode: str = field(init=False, default="manual")

    global_freq: float
    global_volt: float
    boards: dict[int, ManualBoardSettings] = field(default_factory=dict)

    @classmethod
    def from_dict(cls, dict_conf: dict | None) -> "MiningModeManual":
        return cls(
            global_freq=dict_conf["global_freq"],
            global_volt=dict_conf["global_volt"],
            boards={i: ManualBoardSettings.from_dict(dict_conf[i]) for i in dict_conf},
        )

    def as_am_modern(self) -> dict:
        if settings.get("antminer_mining_mode_as_str", False):
            return {"miner-mode": "0"}
        return {"miner-mode": 0}

    def as_elphapex(self) -> dict:
        return {"miner-mode": 0}

    def as_vnish(self) -> dict:
        chains = [b.as_vnish() for b in self.boards.values() if b.freq != 0]
        return {
            "overclock": {
                "chains": chains if chains != [] else None,
                "globals": {
                    "freq": int(self.global_freq),
                    "volt": int(self.global_volt),
                },
            }
        }

    @classmethod
    def from_vnish(cls, web_overclock_settings: dict) -> "MiningModeManual":
        # will raise KeyError if it cant find the settings, values cannot be empty
        voltage = web_overclock_settings["globals"]["volt"]
        freq = web_overclock_settings["globals"]["freq"]
        boards = {
            idx: ManualBoardSettings(
                freq=board["freq"],
                volt=voltage if not board["freq"] == 0 else 0,
            )
            for idx, board in enumerate(web_overclock_settings["chains"])
        }
        return cls(global_freq=freq, global_volt=voltage, boards=boards)

    @classmethod
    def from_epic(cls, epic_conf: dict) -> "MiningModeManual":
        voltage = 0
        freq = 0
        if epic_conf.get("HwConfig") is not None:
            freq = epic_conf["HwConfig"]["Boards Target Clock"][0]["Data"]
        if epic_conf.get("Power Supply Stats") is not None:
            voltage = epic_conf["Power Supply Stats"]["Target Voltage"]
        boards = {}
        if epic_conf.get("HBs") is not None:
            boards = {
                board["Index"]: ManualBoardSettings(
                    freq=board["Core Clock Avg"], volt=board["Input Voltage"]
                )
                for board in epic_conf["HBs"]
            }
        return cls(global_freq=freq, global_volt=voltage, boards=boards)

    def as_mara(self) -> dict:
        return {
            "mode": {
                "work-mode-selector": "Fixed",
                "fixed": {
                    "frequency": str(self.global_freq),
                    "voltage": self.global_volt,
                },
            }
        }


class MiningModeConfig(MinerConfigOption):
    normal = MiningModeNormal
    low = MiningModeLPM
    high = MiningModeHPM
    sleep = MiningModeSleep
    power_tuning = MiningModePowerTune
    hashrate_tuning = MiningModeHashrateTune
    preset = MiningModePreset
    manual = MiningModeManual

    @classmethod
    def default(cls):
        return cls.normal()

    @classmethod
    def from_dict(cls, dict_conf: dict | None):
        if dict_conf is None:
            return cls.default()

        mode = dict_conf.get("mode")
        if mode is None:
            return cls.default()

        cls_attr = getattr(cls, mode)
        if cls_attr is not None:
            return cls_attr().from_dict(dict_conf)

    @classmethod
    def from_am_modern(cls, web_conf: dict):
        if web_conf.get("bitmain-work-mode") is not None:
            work_mode = web_conf["bitmain-work-mode"]
            if work_mode == "":
                return cls.default()
            if int(work_mode) == 0:
                return cls.normal()
            elif int(work_mode) == 1:
                return cls.sleep()
            elif int(work_mode) == 3:
                return cls.low()
        return cls.default()

    @classmethod
    def from_hiveon_modern(cls, web_conf: dict):
        if web_conf.get("bitmain-work-mode") is not None:
            work_mode = web_conf["bitmain-work-mode"]
            if work_mode == "":
                return cls.default()
            if int(work_mode) == 0:
                return cls.normal()
            elif int(work_mode) == 1:
                return cls.sleep()
            elif int(work_mode) == 3:
                return cls.low()
        return cls.default()

    @classmethod
    def from_elphapex(cls, web_conf: dict):
        if web_conf.get("fc-work-mode") is not None:
            work_mode = web_conf["fc-work-mode"]
            if work_mode == "":
                return cls.default()
            if int(work_mode) == 0:
                return cls.normal()
            elif int(work_mode) == 1:
                return cls.sleep()
            elif int(work_mode) == 3:
                return cls.low()
        return cls.default()

    @classmethod
    def from_epic(cls, web_conf: dict):
        try:
            tuner_running = web_conf["PerpetualTune"]["Running"]
            if tuner_running:
                algo_info = web_conf["PerpetualTune"]["Algorithm"]
                if algo_info.get("VoltageOptimizer") is not None:
                    scaling_cfg = None
                    if "Throttle Step" in algo_info["VoltageOptimizer"]:
                        scaling_cfg = ScalingConfig(
                            minimum=algo_info["VoltageOptimizer"].get(
                                "Min Throttle Target"
                            ),
                            step=algo_info["VoltageOptimizer"].get("Throttle Step"),
                        )

                    return cls.hashrate_tuning(
                        hashrate=algo_info["VoltageOptimizer"].get("Target"),
                        algo=TunerAlgo.voltage_optimizer(),
                        scaling=scaling_cfg,
                    )
                elif algo_info.get("BoardTune") is not None:
                    scaling_cfg = None
                    if "Throttle Step" in algo_info["BoardTune"]:
                        scaling_cfg = ScalingConfig(
                            minimum=algo_info["BoardTune"].get("Min Throttle Target"),
                            step=algo_info["BoardTune"].get("Throttle Step"),
                        )

                    return cls.hashrate_tuning(
                        hashrate=algo_info["BoardTune"].get("Target"),
                        algo=TunerAlgo.board_tune(),
                        scaling=scaling_cfg,
                    )
                else:
                    return cls.hashrate_tuning(
                        hashrate=algo_info["ChipTune"].get("Target"),
                        algo=TunerAlgo.chip_tune(),
                    )
            else:
                return MiningModeManual.from_epic(web_conf)
        except KeyError:
            return cls.default()

    @classmethod
    def from_bosminer(cls, toml_conf: dict):
        if toml_conf.get("autotuning") is None:
            return cls.default()
        autotuning_conf = toml_conf["autotuning"]

        if autotuning_conf.get("enabled") is None:
            return cls.default()
        if not autotuning_conf["enabled"]:
            return cls.default()

        if autotuning_conf.get("psu_power_limit") is not None:
            # old autotuning conf
            return cls.power_tuning(
                power=autotuning_conf["psu_power_limit"],
                scaling=ScalingConfig.from_bosminer(toml_conf, mode="power"),
            )
        if autotuning_conf.get("mode") is not None:
            # new autotuning conf
            mode = autotuning_conf["mode"]
            if mode == "power_target":
                if autotuning_conf.get("power_target") is not None:
                    return cls.power_tuning(
                        power=autotuning_conf["power_target"],
                        scaling=ScalingConfig.from_bosminer(toml_conf, mode="power"),
                    )
                return cls.power_tuning(
                    scaling=ScalingConfig.from_bosminer(toml_conf, mode="power"),
                )
            if mode == "hashrate_target":
                if autotuning_conf.get("hashrate_target") is not None:
                    return cls.hashrate_tuning(
                        hashrate=autotuning_conf["hashrate_target"],
                        scaling=ScalingConfig.from_bosminer(toml_conf, mode="hashrate"),
                    )
                return cls.hashrate_tuning(
                    scaling=ScalingConfig.from_bosminer(toml_conf, mode="hashrate"),
                )

    @classmethod
    def from_vnish(cls, web_settings: dict, web_presets: list[dict]):
        try:
            mode_settings = web_settings["miner"]["overclock"]
        except KeyError:
            return cls.default()

        if mode_settings["preset"] == "disabled":
            return MiningModeManual.from_vnish(mode_settings)
        else:
            return MiningModePreset.from_vnish(mode_settings, web_presets)

    @classmethod
    def from_boser(cls, grpc_miner_conf: dict):
        try:
            tuner_conf = grpc_miner_conf["tuner"]
            if not tuner_conf.get("enabled", False):
                return cls.default()
        except LookupError:
            return cls.default()

        if tuner_conf.get("tunerMode") is not None:
            if tuner_conf["tunerMode"] == 1:
                if tuner_conf.get("powerTarget") is not None:
                    return cls.power_tuning(
                        power=tuner_conf["powerTarget"]["watt"],
                        scaling=ScalingConfig.from_boser(grpc_miner_conf, mode="power"),
                    )
                return cls.power_tuning(
                    scaling=ScalingConfig.from_boser(grpc_miner_conf, mode="power")
                )

            if tuner_conf["tunerMode"] == 2:
                if tuner_conf.get("hashrateTarget") is not None:
                    return cls.hashrate_tuning(
                        hashrate=int(tuner_conf["hashrateTarget"]["terahashPerSecond"]),
                        scaling=ScalingConfig.from_boser(
                            grpc_miner_conf, mode="hashrate"
                        ),
                    )
                return cls.hashrate_tuning(
                    scaling=ScalingConfig.from_boser(grpc_miner_conf, mode="hashrate"),
                )

        if tuner_conf.get("powerTarget") is not None:
            return cls.power_tuning(
                power=tuner_conf["powerTarget"]["watt"],
                scaling=ScalingConfig.from_boser(grpc_miner_conf, mode="power"),
            )

        if tuner_conf.get("hashrateTarget") is not None:
            return cls.hashrate_tuning(
                hashrate=int(tuner_conf["hashrateTarget"]["terahashPerSecond"]),
                scaling=ScalingConfig.from_boser(grpc_miner_conf, mode="hashrate"),
            )

        return cls.default()

    @classmethod
    def from_auradine(cls, web_mode: dict):
        try:
            mode_data = web_mode["Mode"][0]
            if mode_data.get("Sleep") == "on":
                return cls.sleep()
            if mode_data.get("Mode") == "normal":
                return cls.normal()
            if mode_data.get("Mode") == "eco":
                return cls.low()
            if mode_data.get("Mode") == "turbo":
                return cls.high()
            if mode_data.get("Ths") is not None:
                return cls.hashrate_tuning(hashrate=mode_data["Ths"])
            if mode_data.get("Power") is not None:
                return cls.power_tuning(power=mode_data["Power"])
        except LookupError:
            return cls.default()

    @classmethod
    def from_mara(cls, web_config: dict):
        try:
            mode = web_config["mode"]["work-mode-selector"]
            if mode == "Fixed":
                fixed_conf = web_config["mode"]["fixed"]
                return cls.manual(
                    global_freq=int(fixed_conf["frequency"]),
                    global_volt=fixed_conf["voltage"],
                )
            elif mode == "Stock":
                return cls.normal()
            elif mode == "Sleep":
                return cls.sleep()
            elif mode == "Auto":
                auto_conf = web_config["mode"]["concorde"]
                auto_mode = auto_conf["mode-select"]
                if auto_mode == "Hashrate":
                    return cls.hashrate_tuning(hashrate=auto_conf["hash-target"])
                elif auto_mode == "PowerTarget":
                    return cls.power_tuning(power=auto_conf["power-target"])
        except LookupError:
            pass
        return cls.default()

    @classmethod
    def from_luxos(cls, rpc_config: dict, rpc_profiles: dict):
        preset_info = MiningModePreset.from_luxos(rpc_config, rpc_profiles)
        return cls.preset(
            active_preset=preset_info.active_preset,
            available_presets=preset_info.available_presets,
        )


MiningMode = TypeVar(
    "MiningMode",
    bound=Union[
        MiningModeNormal,
        MiningModeHPM,
        MiningModeLPM,
        MiningModeSleep,
        MiningModeManual,
        MiningModePowerTune,
        MiningModeHashrateTune,
        MiningModePreset,
    ],
)



from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M33SPlusPlusVG40


class BTMinerM33SPlusPlusVG40(M3X, M33SPlusPlusVG40):
    pass


from pyasic.miners.device.models import M33SPlusPlusVH20


class BTMinerM33SPlusPlusVH20(M3X, M33SPlusPlusVH20):
    pass


from pyasic.miners.device.models import M33SPlusPlusVH30


class BTMinerM33SPlusPlusVH30(M3X, M33SPlusPlusVH30):
    pass



from __future__ import annotations

from .base import MinerAlgoType
from .hashrate import Blake256HashRate
from .hashrate.unit import Blake256Unit


# make this json serializable
class Blake256Algo(MinerAlgoType):
    hashrate: type[Blake256HashRate] = Blake256HashRate
    unit: type[Blake256Unit] = Blake256Unit

    name = "Blake256"



from __future__ import annotations

from typing_extensions import Self

from pyasic.device.algorithm.hashrate.base import AlgoHashRateType
from pyasic.device.algorithm.hashrate.unit.blake256 import Blake256Unit

from .unit import HashUnit


class Blake256HashRate(AlgoHashRateType):
    rate: float
    unit: Blake256Unit = HashUnit.BLAKE256.default

    def into(self, other: Blake256Unit) -> Self:
        return self.__class__(
            rate=self.rate / (other.value / self.unit.value), unit=other
        )



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M63SVK10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SVK10

    expected_chips = 340
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SVK20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SVK20

    expected_chips = 350
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SVK30

    expected_chips = 370
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SVK60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SVK60

    expected_chips = 350
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SVL10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SVL10

    expected_chips = 228
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SVL50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SVL50

    expected_chips = 288
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256


class M63SVL60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M63SVL60

    expected_chips = 288
    expected_fans = 0
    expected_hashboards = 4
    algo = MinerAlgo.SHA256



# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------
import asyncio
import ipaddress
import warnings
from typing import List, Optional, Protocol, Tuple, Type, TypeVar, Union

from pyasic.config import MinerConfig
from pyasic.data import Fan, HashBoard, MinerData
from pyasic.data.device import DeviceInfo
from pyasic.data.error_codes import MinerErrorData
from pyasic.data.pools import PoolMetrics
from pyasic.device.algorithm import MinerAlgoType
from pyasic.device.algorithm.base import GenericAlgo
from pyasic.device.algorithm.hashrate import AlgoHashRate
from pyasic.device.firmware import MinerFirmware
from pyasic.device.makes import MinerMake
from pyasic.device.models import MinerModelType
from pyasic.errors import APIError
from pyasic.logger import logger
from pyasic.miners.data import DataLocations, DataOptions, RPCAPICommand, WebAPICommand


class MinerProtocol(Protocol):
    _rpc_cls: Type = None
    _web_cls: Type = None
    _ssh_cls: Type = None

    ip: str = None
    rpc: _rpc_cls = None
    web: _web_cls = None
    ssh: _ssh_cls = None

    make: MinerMake = None
    raw_model: MinerModelType = None
    firmware: MinerFirmware = None
    algo: type[MinerAlgoType] = GenericAlgo

    expected_hashboards: int = None
    expected_chips: int = None
    expected_fans: int = None

    data_locations: DataLocations = None

    supports_shutdown: bool = False
    supports_power_modes: bool = False
    supports_presets: bool = False
    supports_autotuning: bool = False

    api_ver: str = None
    fw_ver: str = None
    light: bool = None
    config: MinerConfig = None

    def __repr__(self):
        return f"{self.model}: {str(self.ip)}"

    def __lt__(self, other):
        return ipaddress.ip_address(self.ip) < ipaddress.ip_address(other.ip)

    def __gt__(self, other):
        return ipaddress.ip_address(self.ip) > ipaddress.ip_address(other.ip)

    def __eq__(self, other):
        return ipaddress.ip_address(self.ip) == ipaddress.ip_address(other.ip)

    @property
    def model(self) -> str:
        if self.raw_model is not None:
            model_data = [self.raw_model]
        elif self.make is not None:
            model_data = [self.make]
        else:
            model_data = ["Unknown"]
        if self.firmware is not None:
            model_data.append(f"({self.firmware})")
        return " ".join(model_data)

    @property
    def device_info(self) -> DeviceInfo:
        return DeviceInfo(
            make=self.make, model=self.raw_model, firmware=self.firmware, algo=self.algo
        )

    @property
    def api(self):
        return self.rpc

    async def check_light(self) -> bool:
        """Get the status of the fault light as a boolean.

        Returns:
            A boolean value representing the fault light status.
        """
        return await self.get_fault_light()

    async def fault_light_on(self) -> bool:
        """Turn the fault light of the miner on and return success as a boolean.

        Returns:
            A boolean value of the success of turning the light on.
        """
        return False

    async def fault_light_off(self) -> bool:
        """Turn the fault light of the miner off and return success as a boolean.

        Returns:
            A boolean value of the success of turning the light off.
        """
        return False

    async def get_config(self) -> MinerConfig:
        # Not a data gathering function, since this is used for configuration
        """Get the mining configuration of the miner and return it as a [`MinerConfig`][pyasic-umhost.config.MinerConfig].

        Returns:
            A [`MinerConfig`][pyasic-umhost.config.MinerConfig] containing the pool information and mining configuration.
        """
        return MinerConfig()

    async def reboot(self) -> bool:
        """Reboot the miner and return success as a boolean.

        Returns:
            A boolean value of the success of rebooting the miner.
        """
        return False

    async def restart_backend(self) -> bool:
        """Restart the mining process of the miner (bosminer, bmminer, cgminer, etc) and return success as a boolean.

        Returns:
            A boolean value of the success of restarting the mining process.
        """
        return False

    async def send_config(self, config: MinerConfig, user_suffix: str = None) -> None:
        """Set the mining configuration of the miner.

        Parameters:
            config: A [`MinerConfig`][pyasic-umhost.config.MinerConfig] containing the mining config you want to switch the miner to.
            user_suffix: A suffix to append to the username when sending to the miner.
        """
        return None

    async def stop_mining(self) -> bool:
        """Stop the mining process of the miner.

        Returns:
            A boolean value of the success of stopping the mining process.
        """
        return False

    async def resume_mining(self) -> bool:
        """Resume the mining process of the miner.

        Returns:
            A boolean value of the success of resuming the mining process.
        """
        return False

    async def set_power_limit(self, wattage: int) -> bool:
        """Set the power limit to be used by the miner.

        Parameters:
            wattage: The power limit to set on the miner.

        Returns:
            A boolean value of the success of setting the power limit.
        """
        return False

    async def upgrade_firmware(
        self,
        *,
        file: str = None,
        url: str = None,
        version: str = None,
        keep_settings: bool = True,
    ) -> bool:
        """Upgrade the firmware of the miner.

        Parameters:
            file: The file path to the firmware to upgrade from. Must be a valid file path if provided.
            url: The URL to download the firmware from. Must be a valid URL if provided.
            version: The version of the firmware to upgrade to. If None, the version will be inferred from the file or URL.
            keep_settings: Whether to keep the current settings during the upgrade. Defaults to True.

        Returns:
            A boolean value of the success of the firmware upgrade.
        """
        return False

    ##################################################
    ### DATA GATHERING FUNCTIONS (get_{some_data}) ###
    ##################################################

    async def get_mac(self) -> Optional[str]:
        """Get the MAC address of the miner and return it as a string.

        Returns:
            A string representing the MAC address of the miner.
        """
        return await self._get_mac()

    async def get_model(self) -> Optional[str]:
        """Get the model of the miner and return it as a string.

        Returns:
            A string representing the model of the miner.
        """
        return self.model

    async def get_device_info(self) -> Optional[DeviceInfo]:
        """Get device information, including model, make, and firmware.

        Returns:
            A dataclass containing device information.
        """
        return self.device_info

    async def get_api_ver(self) -> Optional[str]:
        """Get the API version of the miner and is as a string.

        Returns:
            API version as a string.
        """
        return await self._get_api_ver()

    async def get_fw_ver(self) -> Optional[str]:
        """Get the firmware version of the miner and is as a string.

        Returns:
            Firmware version as a string.
        """
        return await self._get_fw_ver()

    async def get_version(self) -> Tuple[Optional[str], Optional[str]]:
        """Get the API version and firmware version of the miner and return them as strings.

        Returns:
            A tuple of (API version, firmware version) as strings.
        """
        api_ver = await self.get_api_ver()
        fw_ver = await self.get_fw_ver()
        return api_ver, fw_ver

    async def get_hostname(self) -> Optional[str]:
        """Get the hostname of the miner and return it as a string.

        Returns:
            A string representing the hostname of the miner.
        """
        return await self._get_hostname()

    async def get_hashrate(self) -> Optional[AlgoHashRate]:
        """Get the hashrate of the miner and return it as a float in TH/s.

        Returns:
            Hashrate of the miner in TH/s as a float.
        """
        return await self._get_hashrate()

    async def get_hashboards(self) -> List[HashBoard]:
        """Get hashboard data from the miner in the form of [`HashBoard`][pyasic-umhost.data.HashBoard].

        Returns:
            A [`HashBoard`][pyasic-umhost.data.HashBoard] instance containing hashboard data from the miner.
        """
        return await self._get_hashboards()

    async def get_env_temp(self) -> Optional[float]:
        """Get environment temp from the miner as a float.

        Returns:
            Environment temp of the miner as a float.
        """
        return await self._get_env_temp()

    async def get_wattage(self) -> Optional[int]:
        """Get wattage from the miner as an int.

        Returns:
            Wattage of the miner as an int.
        """
        return await self._get_wattage()

    async def get_voltage(self) -> Optional[float]:
        """Get output voltage of the PSU as a float.

        Returns:
            Output voltage of the PSU as an float.
        """
        return await self._get_voltage()

    async def get_wattage_limit(self) -> Optional[int]:
        """Get wattage limit from the miner as an int.

        Returns:
            Wattage limit of the miner as an int.
        """
        return await self._get_wattage_limit()

    async def get_fans(self) -> List[Fan]:
        """Get fan data from the miner in the form [fan_1, fan_2, fan_3, fan_4].

        Returns:
            A list of fan data.
        """
        return await self._get_fans()

    async def get_fan_psu(self) -> Optional[int]:
        """Get PSU fan speed from the miner.

        Returns:
            PSU fan speed.
        """
        return await self._get_fan_psu()

    async def get_errors(self) -> List[MinerErrorData]:
        """Get a list of the errors the miner is experiencing.

        Returns:
            A list of error classes representing different errors.
        """
        return await self._get_errors()

    async def get_fault_light(self) -> bool:
        """Check the status of the fault light and return on or off as a boolean.

        Returns:
            A boolean value where `True` represents on and `False` represents off.
        """
        return await self._get_fault_light()

    async def get_expected_hashrate(self) -> Optional[AlgoHashRate]:
        """Get the nominal hashrate from factory if available.

        Returns:
            A float value of nominal hashrate in TH/s.
        """
        return await self._get_expected_hashrate()

    async def is_mining(self) -> Optional[bool]:
        """Check whether the miner is mining.

        Returns:
            A boolean value representing if the miner is mining.
        """
        return await self._is_mining()

    async def get_uptime(self) -> Optional[int]:
        """Get the uptime of the miner in seconds.

        Returns:
            The uptime of the miner in seconds.
        """
        return await self._get_uptime()

    async def get_pools(self) -> List[PoolMetrics]:
        """Get the pools information from Miner.

        Returns:
            The pool information of the miner.
        """
        return await self._get_pools()

    async def _get_mac(self) -> Optional[str]:
        pass

    async def _get_api_ver(self) -> Optional[str]:
        pass

    async def _get_fw_ver(self) -> Optional[str]:
        pass

    async def _get_hostname(self) -> Optional[str]:
        pass

    async def _get_hashrate(self) -> Optional[AlgoHashRate]:
        pass

    async def _get_hashboards(self) -> List[HashBoard]:
        return []

    async def _get_env_temp(self) -> Optional[float]:
        pass

    async def _get_wattage(self) -> Optional[int]:
        pass

    async def _get_voltage(self) -> Optional[float]:
        pass

    async def _get_wattage_limit(self) -> Optional[int]:
        pass

    async def _get_fans(self) -> List[Fan]:
        return []

    async def _get_fan_psu(self) -> Optional[int]:
        pass

    async def _get_errors(self) -> List[MinerErrorData]:
        return []

    async def _get_fault_light(self) -> Optional[bool]:
        pass

    async def _get_expected_hashrate(self) -> Optional[AlgoHashRate]:
        pass

    async def _is_mining(self) -> Optional[bool]:
        pass

    async def _get_uptime(self) -> Optional[int]:
        pass

    async def _get_pools(self) -> List[PoolMetrics]:
        pass

    async def _get_data(
        self,
        allow_warning: bool,
        include: List[Union[str, DataOptions]] = None,
        exclude: List[Union[str, DataOptions]] = None,
    ) -> dict:
        # handle include
        if include is not None:
            include = [str(i) for i in include]
        else:
            # everything
            include = [str(enum_value.value) for enum_value in DataOptions]

        # handle exclude
        # prioritized over include, including x and excluding x will exclude x
        if exclude is not None:
            for item in exclude:
                if str(item) in include:
                    include.remove(str(item))

        rpc_multicommand = set()
        web_multicommand = set()
        # create multicommand
        for data_name in include:
            try:
                # get kwargs needed for the _get_xyz function
                fn_args = getattr(self.data_locations, data_name).kwargs

                # keep track of which RPC/Web commands need to be sent
                for arg in fn_args:
                    if isinstance(arg, RPCAPICommand):
                        rpc_multicommand.add(arg.cmd)
                    if isinstance(arg, WebAPICommand):
                        web_multicommand.add(arg.cmd)
            except KeyError as e:
                logger.error(type(e), e, data_name)
                continue

        # create tasks for all commands that need to be sent, or no-op with sleep(0) -> None
        if len(rpc_multicommand) > 0:
            rpc_command_task = asyncio.create_task(
                self.rpc.multicommand(*rpc_multicommand, allow_warning=allow_warning)
            )
        else:
            rpc_command_task = asyncio.create_task(asyncio.sleep(0))
        if len(web_multicommand) > 0:
            web_command_task = asyncio.create_task(
                self.web.multicommand(*web_multicommand, allow_warning=allow_warning)
            )
        else:
            web_command_task = asyncio.create_task(asyncio.sleep(0))

        # make sure the tasks complete
        await asyncio.gather(rpc_command_task, web_command_task)

        # grab data out of the tasks
        web_command_data = web_command_task.result()
        if web_command_data is None:
            web_command_data = {}
        api_command_data = rpc_command_task.result()
        if api_command_data is None:
            api_command_data = {}

        miner_data = {}

        for data_name in include:
            try:
                fn_args = getattr(self.data_locations, data_name).kwargs
                args_to_send = {k.name: None for k in fn_args}
                for arg in fn_args:
                    try:
                        if isinstance(arg, RPCAPICommand):
                            if api_command_data.get("multicommand"):
                                args_to_send[arg.name] = api_command_data[arg.cmd][0]
                            else:
                                args_to_send[arg.name] = api_command_data
                        if isinstance(arg, WebAPICommand):
                            if web_command_data is not None:
                                if web_command_data.get("multicommand"):
                                    args_to_send[arg.name] = web_command_data[arg.cmd]
                                else:
                                    if not web_command_data == {"multicommand": False}:
                                        args_to_send[arg.name] = web_command_data
                    except LookupError:
                        args_to_send[arg.name] = None
            except LookupError:
                continue
            try:
                function = getattr(self, getattr(self.data_locations, data_name).cmd)
                miner_data[data_name] = await function(**args_to_send)
            except Exception as e:
                raise APIError(
                    f"Failed to call {data_name} on {self} while getting data."
                ) from e
        return miner_data

    async def get_data(
        self,
        allow_warning: bool = False,
        include: List[Union[str, DataOptions]] = None,
        exclude: List[Union[str, DataOptions]] = None,
    ) -> MinerData:
        """Get data from the miner in the form of [`MinerData`][pyasic-umhost.data.MinerData].

        Parameters:
            allow_warning: Allow warning when an API command fails.
            include: Names of data items you want to gather. Defaults to all data.
            exclude: Names of data items to exclude.  Exclusion happens after considering included items.

        Returns:
            A [`MinerData`][pyasic-umhost.data.MinerData] instance containing data from the miner.
        """
        data = MinerData(
            ip=str(self.ip),
            device_info=self.device_info,
            expected_chips=(
                self.expected_chips * self.expected_hashboards
                if self.expected_chips is not None
                else 0
            ),
            expected_hashboards=self.expected_hashboards,
            expected_fans=self.expected_fans,
            hashboards=[
                HashBoard(slot=i, expected_chips=self.expected_chips)
                for i in range(
                    self.expected_hashboards
                    if self.expected_hashboards is not None
                    else 0
                )
            ],
        )

        gathered_data = await self._get_data(
            allow_warning=allow_warning, include=include, exclude=exclude
        )
        for item in gathered_data:
            if gathered_data[item] is not None:
                setattr(data, item, gathered_data[item])

        return data


class BaseMiner(MinerProtocol):
    def __init__(self, ip: str) -> None:
        self.ip = ip

        if self.expected_chips is None and self.raw_model is not None:
            warnings.warn(
                f"Unknown chip count for miner type {self.raw_model}, "
                f"please open an issue on GitHub (https://github.com/UpstreamData/pyasic)."
            )

        # interfaces
        if self._rpc_cls is not None:
            self.rpc = self._rpc_cls(ip)
        if self._web_cls is not None:
            self.web = self._web_cls(ip)
        if self._ssh_cls is not None:
            self.ssh = self._ssh_cls(ip)


AnyMiner = TypeVar("AnyMiner", bound=BaseMiner)



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M50SPlusVH30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVH30

    expected_chips = 172
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVH40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVH40

    expected_chips = 180
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVJ30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVJ30

    expected_chips = 156
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVJ40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVJ40

    expected_chips = 164
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVJ60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVJ60

    expected_chips = 164
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVK10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVK10

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVK20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVK20

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVK30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVK30

    expected_chips = 123
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVL10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVL10

    expected_chips = 82
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVL20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVL20

    expected_chips = 86
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M50SPlusVL30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M50SPlusVL30

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M30SV10


class BTMinerM30SV10(M3X, M30SV10):
    pass


from pyasic.miners.device.models import M30SV20


class BTMinerM30SV20(M3X, M30SV20):
    pass


from pyasic.miners.device.models import M30SV30


class BTMinerM30SV30(M3X, M30SV30):
    pass


from pyasic.miners.device.models import M30SV40


class BTMinerM30SV40(M3X, M30SV40):
    pass


from pyasic.miners.device.models import M30SV50


class BTMinerM30SV50(M3X, M30SV50):
    pass


from pyasic.miners.device.models import M30SV60


class BTMinerM30SV60(M3X, M30SV60):
    pass


from pyasic.miners.device.models import M30SV70


class BTMinerM30SV70(M3X, M30SV70):
    pass


from pyasic.miners.device.models import M30SV80


class BTMinerM30SV80(M3X, M30SV80):
    pass


from pyasic.miners.device.models import M30SVE10


class BTMinerM30SVE10(M3X, M30SVE10):
    pass


from pyasic.miners.device.models import M30SVE20


class BTMinerM30SVE20(M3X, M30SVE20):
    pass


from pyasic.miners.device.models import M30SVE30


class BTMinerM30SVE30(M3X, M30SVE30):
    pass


from pyasic.miners.device.models import M30SVE40


class BTMinerM30SVE40(M3X, M30SVE40):
    pass


from pyasic.miners.device.models import M30SVE50


class BTMinerM30SVE50(M3X, M30SVE50):
    pass


from pyasic.miners.device.models import M30SVE60


class BTMinerM30SVE60(M3X, M30SVE60):
    pass


from pyasic.miners.device.models import M30SVE70


class BTMinerM30SVE70(M3X, M30SVE70):
    pass


from pyasic.miners.device.models import M30SVF10


class BTMinerM30SVF10(M3X, M30SVF10):
    pass


from pyasic.miners.device.models import M30SVF20


class BTMinerM30SVF20(M3X, M30SVF20):
    pass


from pyasic.miners.device.models import M30SVF30


class BTMinerM30SVF30(M3X, M30SVF30):
    pass


from pyasic.miners.device.models import M30SVG10


class BTMinerM30SVG10(M3X, M30SVG10):
    pass


from pyasic.miners.device.models import M30SVG20


class BTMinerM30SVG20(M3X, M30SVG20):
    pass


from pyasic.miners.device.models import M30SVG30


class BTMinerM30SVG30(M3X, M30SVG30):
    pass


from pyasic.miners.device.models import M30SVG40


class BTMinerM30SVG40(M3X, M30SVG40):
    pass


from pyasic.miners.device.models import M30SVH10


class BTMinerM30SVH10(M3X, M30SVH10):
    pass


from pyasic.miners.device.models import M30SVH20


class BTMinerM30SVH20(M3X, M30SVH20):
    pass


from pyasic.miners.device.models import M30SVH30


class BTMinerM30SVH30(M3X, M30SVH30):
    pass


from pyasic.miners.device.models import M30SVH40


class BTMinerM30SVH40(M3X, M30SVH40):
    pass


from pyasic.miners.device.models import M30SVH50


class BTMinerM30SVH50(M3X, M30SVH50):
    pass


from pyasic.miners.device.models import M30SVH60


class BTMinerM30SVH60(M3X, M30SVH60):
    pass


from pyasic.miners.device.models import M30SVI20


class BTMinerM30SVI20(M3X, M30SVI20):
    pass


from pyasic.miners.device.models import M30SVJ30


class BTMinerM30SVJ30(M3X, M30SVJ30):
    pass



from pyasic import settings
from pyasic.ssh.base import BaseSSH


class AntminerModernSSH(BaseSSH):
    """
    Initialize an AntminerModernSSH instance.

    Args:
        ip (str): The IP address of the Antminer device.
    """

    def __init__(self, ip: str):
        super().__init__(ip)
        self.pwd = settings.get("default_antminer_ssh_password", "root")
        self.username = "miner"



# ------------------------------------------------------------------------------
#  Copyright 2022 Upstream Data Inc                                            -
#                                                                              -
#  Licensed under the Apache License, Version 2.0 (the "License");             -
#  you may not use this file except in compliance with the License.            -
#  You may obtain a copy of the License at                                     -
#                                                                              -
#      http://www.apache.org/licenses/LICENSE-2.0                              -
#                                                                              -
#  Unless required by applicable law or agreed to in writing, software         -
#  distributed under the License is distributed on an "AS IS" BASIS,           -
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    -
#  See the License for the specific language governing permissions and         -
#  limitations under the License.                                              -
# ------------------------------------------------------------------------------
from typing import Optional

from pyasic import APIError
from pyasic.miners.backends import AvalonMiner
from pyasic.miners.data import (
    DataFunction,
    DataLocations,
    DataOptions,
    RPCAPICommand,
    WebAPICommand,
)
from pyasic.miners.device.models import AvalonNano3
from pyasic.web.avalonminer import AvalonMinerWebAPI

AVALON_NANO_DATA_LOC = DataLocations(
    **{
        str(DataOptions.MAC): DataFunction(
            "_get_mac",
            [WebAPICommand("web_minerinfo", "get_minerinfo")],
        ),
        str(DataOptions.API_VERSION): DataFunction(
            "_get_api_ver",
            [RPCAPICommand("rpc_version", "version")],
        ),
        str(DataOptions.FW_VERSION): DataFunction(
            "_get_fw_ver",
            [RPCAPICommand("rpc_version", "version")],
        ),
        str(DataOptions.HASHRATE): DataFunction(
            "_get_hashrate",
            [RPCAPICommand("rpc_devs", "devs")],
        ),
        str(DataOptions.EXPECTED_HASHRATE): DataFunction(
            "_get_expected_hashrate",
            [RPCAPICommand("rpc_stats", "stats")],
        ),
        str(DataOptions.HASHBOARDS): DataFunction(
            "_get_hashboards",
            [RPCAPICommand("rpc_stats", "stats")],
        ),
        str(DataOptions.ENVIRONMENT_TEMP): DataFunction(
            "_get_env_temp",
            [RPCAPICommand("rpc_stats", "stats")],
        ),
        str(DataOptions.WATTAGE_LIMIT): DataFunction(
            "_get_wattage_limit",
            [RPCAPICommand("rpc_stats", "stats")],
        ),
        str(DataOptions.WATTAGE): DataFunction(
            "_get_wattage",
            [RPCAPICommand("rpc_stats", "stats")],
        ),
        str(DataOptions.FANS): DataFunction(
            "_get_fans",
            [RPCAPICommand("rpc_stats", "stats")],
        ),
        str(DataOptions.FAULT_LIGHT): DataFunction(
            "_get_fault_light",
            [RPCAPICommand("rpc_stats", "stats")],
        ),
        str(DataOptions.UPTIME): DataFunction(
            "_get_uptime",
            [RPCAPICommand("rpc_stats", "stats")],
        ),
        str(DataOptions.POOLS): DataFunction(
            "_get_pools",
            [RPCAPICommand("rpc_pools", "pools")],
        ),
    }
)


class CGMinerAvalonNano3(AvalonMiner, AvalonNano3):
    _web_cls = AvalonMinerWebAPI
    web: AvalonMinerWebAPI

    data_locations = AVALON_NANO_DATA_LOC

    async def _get_mac(self, web_minerinfo: dict) -> Optional[dict]:
        if web_minerinfo is None:
            try:
                web_minerinfo = await self.web.minerinfo()
            except APIError:
                pass

        if web_minerinfo is not None:
            try:
                mac = web_minerinfo.get("mac")
                if mac is not None:
                    return mac.upper()
            except (KeyError, ValueError):
                pass



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M30SV10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SV10

    expected_chips = 148
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SV20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SV20

    expected_chips = 156
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SV30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SV30

    expected_chips = 164
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SV40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SV40

    expected_chips = 172
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SV50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SV50

    expected_chips = 156
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SV60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SV60

    expected_chips = 164
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SV70(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SV70

    expected_chips = None
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SV80(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SV80

    expected_chips = 129
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVE10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVE10

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVE20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVE20

    expected_chips = 111
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVE30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVE30

    expected_chips = 117
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVE40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVE40

    expected_chips = 123
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVE50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVE50

    expected_chips = 129
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVE60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVE60

    expected_chips = None
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVE70(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVE70

    expected_chips = None
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVF10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVF10

    expected_chips = 70
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVF20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVF20

    expected_chips = 74
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVF30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVF30

    expected_chips = 78
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVG10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVG10

    expected_chips = 66
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVG20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVG20

    expected_chips = 70
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVG30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVG30

    expected_chips = 74
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVG40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVG40

    expected_chips = 78
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVH10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVH10

    expected_chips = 64
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVH20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVH20

    expected_chips = 66
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVH30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVH30

    expected_chips = None
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVH40(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVH40

    expected_chips = 64
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVH50(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVH50

    expected_chips = 66
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVH60(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVH60

    expected_chips = 70
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVI20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVI20

    expected_chips = 70
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M30SVJ30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M30SVJ30

    expected_chips = 105
    expected_fans = 2
    expected_hashboards = 3
    algo = MinerAlgo.SHA256



from pyasic.device.algorithm import MinerAlgo
from pyasic.device.models import MinerModel
from pyasic.miners.device.makes import WhatsMinerMake


class M33V10(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M33V10

    expected_chips = 33
    expected_fans = 0
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M33V20(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M33V20

    expected_chips = 62
    expected_fans = 0
    expected_hashboards = 3
    algo = MinerAlgo.SHA256


class M33V30(WhatsMinerMake):
    raw_model = MinerModel.WHATSMINER.M33V30

    expected_chips = 66
    expected_fans = 0
    expected_hashboards = 3
    algo = MinerAlgo.SHA256
